@@ -87,8 +87,10 @@ extern DescriptorReqFlags DescriptorRequirementsBitsFromFormat(VkFormat fmt);
 struct DescriptorRequirement {
     DescriptorReqFlags reqs;
     bool is_writable;
-    // Copy from StageState.interface_var. It combines from plural shader stages. The index of array is index of image.
+    // Copy from StageState.InterfaceVariable. It combines from plural shader stages. The index of array is index of image.
     std::vector<layer_data::unordered_set<SamplerUsedByImage>> samplers_used_by_image;
+    // For storage images - list of < OpImageWrite : Texel component length >
+    std::vector<std::pair<Instruction, uint32_t>> write_without_formats_component_count_list;
     DescriptorRequirement() : reqs(0), is_writable(false) {}
 };
 
@@ -96,15 +98,17 @@ inline bool operator==(const DescriptorRequirement &a, const DescriptorRequireme
 
 inline bool operator<(const DescriptorRequirement &a, const DescriptorRequirement &b) noexcept { return a.reqs < b.reqs; }
 
+// < binding index (of descriptor set) : meta data >
 typedef std::map<uint32_t, DescriptorRequirement> BindingReqMap;
 
 struct PipelineStageState {
     std::shared_ptr<const SHADER_MODULE_STATE> module_state;
     const safe_VkPipelineShaderStageCreateInfo *create_info;
     VkShaderStageFlagBits stage_flag;
-    spirv_inst_iter entrypoint;
-    layer_data::unordered_set<uint32_t> accessible_ids;
-    using DescriptorUse = std::pair<DescriptorSlot, interface_var>;
+    layer_data::optional<Instruction> entrypoint;
+    // Used to map each descriptor binding to the OpVariable decorated with it
+    // <(set, binding), OpVariable>
+    using DescriptorUse = std::pair<DescriptorSlot, InterfaceVariable>;
     std::vector<DescriptorUse> descriptor_uses;
     bool has_writable_descriptor;
     bool has_atomic_descriptor;
@@ -188,6 +192,14 @@ class PIPELINE_STATE : public BASE_NODE {
     // Additional metadata needed by pipeline_state initialization and validation
     using StageStateVec = std::vector<PipelineStageState>;
     const StageStateVec stage_state;
+    bool HasShaderStage(VkShaderStageFlagBits stage_flag) const {
+        for (const auto &state : stage_state) {
+            if (state.stage_flag == stage_flag) {
+                return true;
+            }
+        }
+        return false;
+    }
 
     const layer_data::unordered_set<uint32_t> fragmentShader_writable_output_location_list;
 
@@ -201,26 +213,31 @@ class PIPELINE_STATE : public BASE_NODE {
 
     // Flag of which shader stages are active for this pipeline
     const uint32_t active_shaders = 0;
-    const VkPrimitiveTopology topology_at_rasterizer;
+    const VkPrimitiveTopology topology_at_rasterizer = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
     const bool uses_shader_module_id;
+    const bool descriptor_buffer_mode = false;
+    const bool uses_pipeline_robustness;
+
+    CreateShaderModuleStates *csm_states = nullptr;
 
     // Executable or legacy pipeline
     PIPELINE_STATE(const ValidationStateTracker *state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
-                   std::shared_ptr<const RENDER_PASS_STATE> &&rpstate, std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout);
+                   std::shared_ptr<const RENDER_PASS_STATE> &&rpstate, std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout,
+                   CreateShaderModuleStates *csm_states = nullptr);
 
     // Compute pipeline
     PIPELINE_STATE(const ValidationStateTracker *state_data, const VkComputePipelineCreateInfo *pCreateInfo,
-                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout);
+                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout, CreateShaderModuleStates *csm_states = nullptr);
 
     PIPELINE_STATE(const ValidationStateTracker *state_data, const VkRayTracingPipelineCreateInfoKHR *pCreateInfo,
-                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout);
+                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout, CreateShaderModuleStates *csm_states = nullptr);
 
     PIPELINE_STATE(const ValidationStateTracker *state_data, const VkRayTracingPipelineCreateInfoNV *pCreateInfo,
-                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout);
+                   std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout, CreateShaderModuleStates *csm_states = nullptr);
 
     // Executable pipeline with all state defined within linked pipeline libraries
     PIPELINE_STATE(const ValidationStateTracker *state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
-                   std::shared_ptr<const RENDER_PASS_STATE> &&rpstate = {});
+                   std::shared_ptr<const RENDER_PASS_STATE> &&rpstate = {}, CreateShaderModuleStates *csm_states = nullptr);
 
     VkPipeline pipeline() const { return handle_.Cast<VkPipeline>(); }
 
@@ -257,7 +274,7 @@ class PIPELINE_STATE : public BASE_NODE {
         }
     }
 
-    bool IsGraphicsLibrary() const { return graphics_lib_type != static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0); }
+    bool IsGraphicsLibrary() const { return !HasFullState(); }
     bool HasFullState() const { return vertex_input_state && pre_raster_state && fragment_shader_state && fragment_output_state; }
 
     template <VkGraphicsPipelineLibraryFlagBitsEXT type_flag>
@@ -268,6 +285,8 @@ class PIPELINE_STATE : public BASE_NODE {
         return {};
     }
 
+    std::shared_ptr<const SHADER_MODULE_STATE> GetSubStateShader(VkShaderStageFlagBits state) const;
+
     template <VkGraphicsPipelineLibraryFlagBitsEXT type_flag>
     static inline typename SubStateTraits<type_flag>::type GetLibSubState(const ValidationStateTracker &state,
                                                                           const VkPipelineLibraryCreateInfoKHR &link_info) {
@@ -431,6 +450,15 @@ class PIPELINE_STATE : public BASE_NODE {
         return {};
     }
 
+    std::shared_ptr<const SHADER_MODULE_STATE> GetShaderModuleState(VkShaderStageFlagBits stage) {
+        for (auto &s : stage_state) {
+            if (s.stage_flag == stage) {
+                return s.module_state;
+            }
+        }
+        return {};
+    }
+
     const safe_VkPipelineDynamicStateCreateInfo *DynamicState() const {
         // TODO Each library can contain its own dynamic state (apparently?). Which one should be returned here? Union?
         return create_info.graphics.pDynamicState;
@@ -474,7 +502,8 @@ class PIPELINE_STATE : public BASE_NODE {
     const void *PNext() const { return create_info.graphics.pNext; }
 
     static ActiveSlotMap GetActiveSlots(const StageStateVec &stage_states);
-    static StageStateVec GetStageStates(const ValidationStateTracker &state_data, const PIPELINE_STATE &pipe_state);
+    static StageStateVec GetStageStates(const ValidationStateTracker &state_data, const PIPELINE_STATE &pipe_state,
+                                        CreateShaderModuleStates *csm_states);
 
     // Return true if for a given PSO, the given state enum is dynamic, else return false
     bool IsDynamic(const VkDynamicState state) const {
@@ -492,17 +521,17 @@ class PIPELINE_STATE : public BASE_NODE {
                                                                     const safe_VkGraphicsPipelineCreateInfo &create_info);
     static std::shared_ptr<PreRasterState> CreatePreRasterState(const PIPELINE_STATE &p, const ValidationStateTracker &state,
                                                                 const safe_VkGraphicsPipelineCreateInfo &create_info,
-                                                                std::shared_ptr<const RENDER_PASS_STATE> rp);
+                                                                const std::shared_ptr<const RENDER_PASS_STATE> &rp);
     static std::shared_ptr<FragmentShaderState> CreateFragmentShaderState(const PIPELINE_STATE &p,
                                                                           const ValidationStateTracker &state,
                                                                           const VkGraphicsPipelineCreateInfo &create_info,
                                                                           const safe_VkGraphicsPipelineCreateInfo &safe_create_info,
-                                                                          std::shared_ptr<const RENDER_PASS_STATE> rp);
+                                                                          const std::shared_ptr<const RENDER_PASS_STATE> &rp);
     static std::shared_ptr<FragmentOutputState> CreateFragmentOutputState(const PIPELINE_STATE &p,
                                                                           const ValidationStateTracker &state,
                                                                           const VkGraphicsPipelineCreateInfo &create_info,
                                                                           const safe_VkGraphicsPipelineCreateInfo &safe_create_info,
-                                                                          std::shared_ptr<const RENDER_PASS_STATE> rp);
+                                                                          const std::shared_ptr<const RENDER_PASS_STATE> &rp);
 
     // Merged layouts
     std::shared_ptr<const PIPELINE_LAYOUT_STATE> merged_graphics_layout;
@@ -628,9 +657,15 @@ struct LAST_BOUND_STATE {
     VkPipelineLayout pipeline_layout{VK_NULL_HANDLE};
     std::shared_ptr<cvdescriptorset::DescriptorSet> push_descriptor_set;
 
+    struct DescriptorBufferBinding {
+        uint32_t index{0};
+        VkDeviceSize offset{0};
+    };
     // Ordered bound set tracking where index is set# that given set is bound to
     struct PER_SET {
         std::shared_ptr<cvdescriptorset::DescriptorSet> bound_descriptor_set;
+        std::optional<DescriptorBufferBinding> bound_descriptor_buffer;
+
         // one dynamic offset per dynamic descriptor bound to this CB
         std::vector<uint32_t> dynamicOffsets;
         PipelineLayoutCompatId compat_id_for_set{0};
@@ -640,6 +675,12 @@ struct LAST_BOUND_STATE {
         uint64_t validated_set_change_count{~0ULL};
         uint64_t validated_set_image_layout_change_count{~0ULL};
         BindingReqMap validated_set_binding_req_map;
+
+        void Reset() {
+            bound_descriptor_set.reset();
+            bound_descriptor_buffer.reset();
+            dynamicOffsets.clear();
+        }
     };
 
     std::vector<PER_SET> per_set;
@@ -651,16 +692,22 @@ struct LAST_BOUND_STATE {
     inline bool IsUsing() const { return pipeline_state ? true : false; }
 };
 
-static inline bool CompatForSet(uint32_t set, const LAST_BOUND_STATE &a, const std::vector<PipelineLayoutCompatId> &b) {
-    bool result = (set < a.per_set.size()) && (set < b.size()) && (*(a.per_set[set].compat_id_for_set) == *b[set]);
-    return result;
+static inline bool IsBoundSetCompat(uint32_t set, const LAST_BOUND_STATE &last_bound,
+                                    const PIPELINE_LAYOUT_STATE &pipeline_layout) {
+    if ((set >= last_bound.per_set.size()) || (set >= pipeline_layout.set_compat_ids.size())) {
+        return false;
+    }
+    return (*(last_bound.per_set[set].compat_id_for_set) == *(pipeline_layout.set_compat_ids[set]));
 }
 
-static inline bool CompatForSet(uint32_t set, const PIPELINE_LAYOUT_STATE *a, const PIPELINE_LAYOUT_STATE *b) {
-    // Intentionally have a result variable to simplify debugging
-    bool result = a && b && (set < a->compat_for_set.size()) && (set < b->compat_for_set.size()) &&
-                  (a->compat_for_set[set] == b->compat_for_set[set]);
-    return result;
+static inline bool IsPipelineLayoutSetCompat(uint32_t set, const PIPELINE_LAYOUT_STATE *a, const PIPELINE_LAYOUT_STATE *b) {
+    if (!a || !b) {
+        return false;
+    }
+    if ((set >= a->set_compat_ids.size()) || (set >= b->set_compat_ids.size())) {
+        return false;
+    }
+    return a->set_compat_ids[set] == b->set_compat_ids[set];
 }
 
 enum LvlBindPoint {
