@@ -30,6 +30,7 @@
 #include <algorithm>
 #include <array>
 #include <chrono>
+#include <cstdlib>
 #include <memory>
 #include <mutex>
 #include <thread>
@@ -142,12 +143,10 @@ TEST_F(VkPositiveLayerTest, TestDestroyFreeNullHandles) {
 TEST_F(VkPositiveLayerTest, Maintenance1Tests) {
     TEST_DESCRIPTION("Validate various special cases for the Maintenance1_KHR extension");
 
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
-    if (DeviceExtensionSupported(gpu(), nullptr, VK_KHR_MAINTENANCE_1_EXTENSION_NAME)) {
-        m_device_extension_names.push_back(VK_KHR_MAINTENANCE_1_EXTENSION_NAME);
-    } else {
-        printf("%s Maintenance1 Extension not supported, skipping tests\n", kSkipPrefix);
-        return;
+    AddRequiredExtensions(VK_KHR_MAINTENANCE_1_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
 
@@ -163,12 +162,10 @@ TEST_F(VkPositiveLayerTest, ValidStructPNext) {
     TEST_DESCRIPTION("Verify that a valid pNext value is handled correctly");
 
     // Positive test to check parameter_validation and unique_objects support for NV_dedicated_allocation
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
-    if (DeviceExtensionSupported(gpu(), nullptr, VK_NV_DEDICATED_ALLOCATION_EXTENSION_NAME)) {
-        m_device_extension_names.push_back(VK_NV_DEDICATED_ALLOCATION_EXTENSION_NAME);
-    } else {
-        printf("%s VK_NV_DEDICATED_ALLOCATION_EXTENSION_NAME Extension not supported, skipping test\n", kSkipPrefix);
-        return;
+    AddRequiredExtensions(VK_NV_DEDICATED_ALLOCATION_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
 
@@ -221,7 +218,7 @@ TEST_F(VkPositiveLayerTest, SurfacelessQueryTest) {
         GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
 
-    if (IsPlatform(kMockICD) || DeviceSimulation()) {
+    if (IsPlatform(kMockICD)) {
         GTEST_SKIP() << "Test not supported by MockICD";
     }
 
@@ -239,14 +236,12 @@ TEST_F(VkPositiveLayerTest, SurfacelessQueryTest) {
 
 TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
     TEST_DESCRIPTION("Ensure parameter_validation_layer correctly captures physical device features");
-    if (InstanceExtensionSupported(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME)) {
-        m_instance_extension_names.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    } else {
-        printf("%s Did not find VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME; skipped.\n", kSkipPrefix);
-        return;
-    }
 
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
+    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
 
     VkResult err;
 
@@ -291,7 +286,7 @@ TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
         ASSERT_VK_SUCCESS(err);
         vk::DestroySampler(device, sampler, nullptr);
     } else {
-        printf("%s Feature samplerAnisotropy not enabled;  parameter_layer check skipped.\n", kSkipPrefix);
+        printf("Feature samplerAnisotropy not enabled;  parameter_layer check skipped.\n");
     }
 
     // Verify the core validation layer has captured the physical device features by creating a a query pool.
@@ -305,7 +300,7 @@ TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
 
         vk::DestroyQueryPool(device, query_pool, nullptr);
     } else {
-        printf("%s Feature pipelineStatisticsQuery not enabled;  core_validation_layer check skipped.\n", kSkipPrefix);
+        printf("Feature pipelineStatisticsQuery not enabled;  core_validation_layer check skipped.\n");
     }
 
     vk::DestroyDevice(device, nullptr);
@@ -317,27 +312,6 @@ TEST_F(VkPositiveLayerTest, ApiVersionZero) {
     ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
 }
 
-TEST_F(VkPositiveLayerTest, RayTracingPipelineNV) {
-    TEST_DESCRIPTION("Test VK_NV_ray_tracing.");
-
-    AddRequiredExtensions(VK_NV_RAY_TRACING_EXTENSION_NAME);
-    ASSERT_NO_FATAL_FAILURE(InitFramework());
-    if (!AreRequiredExtensionsEnabled()) {
-        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported.";
-    }
-
-    auto rtnv_props = LvlInitStruct<VkPhysicalDeviceRayTracingPropertiesNV>();
-    GetPhysicalDeviceProperties2(rtnv_props);
-    if (rtnv_props.maxDescriptorSetAccelerationStructures < 1) {
-        GTEST_SKIP() << "VkPhysicalDeviceRayTracingPropertiesNV::maxDescriptorSetAccelerationStructures < 1";
-    }
-
-    ASSERT_NO_FATAL_FAILURE(InitState());
-
-    auto ignore_update = [](CreateNVRayTracingPipelineHelper &helper) {};
-    CreateNVRayTracingPipelineHelper::OneshotPositiveTest(*this, ignore_update);
-}
-
 TEST_F(VkPositiveLayerTest, TestPhysicalDeviceSurfaceSupport) {
     TEST_DESCRIPTION("Test if physical device supports surface.");
     SetTargetApiVersion(VK_API_VERSION_1_1);
@@ -350,8 +324,7 @@ TEST_F(VkPositiveLayerTest, TestPhysicalDeviceSurfaceSupport) {
 
     ASSERT_NO_FATAL_FAILURE(InitState());
     if (!InitSurface()) {
-        printf("%s Cannot create surface, skipping test\n", kSkipPrefix);
-        return;
+        GTEST_SKIP() << "Cannot create surface";
     }
 
     VkBool32 supported;
@@ -368,27 +341,15 @@ TEST_F(VkPositiveLayerTest, ModifyPnext) {
 
     SetTargetApiVersion(VK_API_VERSION_1_2);
 
-    if (InstanceExtensionSupported(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME)) {
-        m_instance_extension_names.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    } else {
-        printf("%s Extension %s is not supported.\n", kSkipPrefix, VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-        return;
+    AddRequiredExtensions(VK_NV_FRAGMENT_SHADING_RATE_ENUMS_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
     if (DeviceValidationVersion() < VK_API_VERSION_1_2) {
         GTEST_SKIP() << "At least Vulkan version 1.2 is required";
     }
 
-    if (DeviceExtensionSupported(gpu(), nullptr, VK_NV_FRAGMENT_SHADING_RATE_ENUMS_EXTENSION_NAME)) {
-        m_device_extension_names.push_back(VK_KHR_MAINTENANCE_2_EXTENSION_NAME);
-        m_device_extension_names.push_back(VK_KHR_MULTIVIEW_EXTENSION_NAME);
-        m_device_extension_names.push_back(VK_KHR_CREATE_RENDERPASS_2_EXTENSION_NAME);
-        m_device_extension_names.push_back(VK_KHR_FRAGMENT_SHADING_RATE_EXTENSION_NAME);
-        m_device_extension_names.push_back(VK_NV_FRAGMENT_SHADING_RATE_ENUMS_EXTENSION_NAME);
-    } else {
-        printf("%s test requires %s\n", kSkipPrefix, VK_NV_FRAGMENT_SHADING_RATE_ENUMS_EXTENSION_NAME);
-    }
-
     auto shading = LvlInitStruct<VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV>();
     shading.maxFragmentShadingRateInvocationCount = static_cast<VkSampleCountFlagBits>(0);
     auto props = LvlInitStruct<VkPhysicalDeviceProperties2>(&shading);
@@ -400,22 +361,13 @@ TEST_F(VkPositiveLayerTest, HostQueryResetSuccess) {
     // This is a positive test. No failures are expected.
     TEST_DESCRIPTION("Use vkResetQueryPoolEXT normally");
 
-    if (!InstanceExtensionSupported(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME)) {
-        printf("%s Did not find required instance extension %s; skipped.\n", kSkipPrefix,
-               VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-        return;
+    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_HOST_QUERY_RESET_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
 
-    m_instance_extension_names.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
-
-    if (!DeviceExtensionSupported(gpu(), nullptr, VK_EXT_HOST_QUERY_RESET_EXTENSION_NAME)) {
-        printf("%s Extension %s not supported by device; skipped.\n", kSkipPrefix, VK_EXT_HOST_QUERY_RESET_EXTENSION_NAME);
-        return;
-    }
-
-    m_device_extension_names.push_back(VK_EXT_HOST_QUERY_RESET_EXTENSION_NAME);
-
     VkPhysicalDeviceHostQueryResetFeaturesEXT host_query_reset_features =
         LvlInitStruct<VkPhysicalDeviceHostQueryResetFeaturesEXT>();
     host_query_reset_features.hostQueryReset = VK_TRUE;
@@ -459,13 +411,10 @@ TEST_F(VkPositiveLayerTest, UseFirstQueueUnqueried) {
 #if !defined(ANDROID)
 TEST_F(VkPositiveLayerTest, GetDevProcAddrNullPtr) {
     TEST_DESCRIPTION("Call GetDeviceProcAddr on an enabled instance extension expecting nullptr");
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
-
-    if (InstanceExtensionSupported(VK_KHR_SURFACE_EXTENSION_NAME)) {
-        m_instance_extension_names.push_back(VK_KHR_SURFACE_EXTENSION_NAME);
-    } else {
-        printf("%s %s not supported, skipping test\n", kSkipPrefix, VK_KHR_SURFACE_EXTENSION_NAME);
-        return;
+    AddRequiredExtensions(VK_KHR_SURFACE_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
 
@@ -579,8 +528,7 @@ TEST_F(VkPositiveLayerTest, QueueThreading) {
     }
     // Test randomly fails with VK_TIMEOUT, most likely a driver bug
     if (IsDriver(VK_DRIVER_ID_AMD_PROPRIETARY)) {
-        printf("%s Test does not run on AMD proprietary driver, skipping tests\n", kSkipPrefix);
-        return;
+        GTEST_SKIP() << "Test does not run on AMD proprietary driver";
     }
     ASSERT_NO_FATAL_FAILURE(InitState());
 
@@ -650,8 +598,7 @@ TEST_F(VkPositiveLayerTest, TestAcquiringSwapchainImages) {
 
     ASSERT_NO_FATAL_FAILURE(InitState(nullptr, nullptr, VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT));
     if (!InitSwapchain()) {
-        printf("%s Cannot create surface or swapchain, skipping test\n", kSkipPrefix);
-        return;
+        GTEST_SKIP() << "Cannot create surface or swapchain";
     }
 
     VkSemaphoreCreateInfo semaphore_create_info = LvlInitStruct<VkSemaphoreCreateInfo>();
@@ -667,8 +614,7 @@ TEST_F(VkPositiveLayerTest, TestAcquiringSwapchainImages) {
     vk::GetSwapchainImagesKHR(device(), m_swapchain, &swapchain_images_count, swapchain_images.data());
 
     uint32_t image_index = 0;
-    vk::AcquireNextImageKHR(device(), m_swapchain, std::numeric_limits<uint64_t>::max(), acquire_semaphore, VK_NULL_HANDLE,
-                            &image_index);
+    vk::AcquireNextImageKHR(device(), m_swapchain, kWaitTimeout, acquire_semaphore, VK_NULL_HANDLE, &image_index);
 
     m_commandBuffer->begin();
 
@@ -717,53 +663,6 @@ TEST_F(VkPositiveLayerTest, TestAcquiringSwapchainImages) {
     vk::DestroySemaphore(device(), acquire_semaphore, nullptr);
 }
 
-TEST_F(VkPositiveLayerTest, ValidateGetAccelerationStructureBuildSizes) {
-    TEST_DESCRIPTION("Test enabled features for GetAccelerationStructureBuildSizes");
-
-    SetTargetApiVersion(VK_API_VERSION_1_1);
-    AddRequiredExtensions(VK_KHR_ACCELERATION_STRUCTURE_EXTENSION_NAME);
-
-    ASSERT_NO_FATAL_FAILURE(InitFramework(m_errorMonitor));
-
-    // Crashes without any warnings
-    if (IsDriver(VK_DRIVER_ID_AMD_PROPRIETARY)) {
-        printf("%s Test does not run on AMD proprietary driver, skipping tests\n", kSkipPrefix);
-        return;
-    }
-
-    if (!AreRequiredExtensionsEnabled()) {
-        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
-    }
-    if (DeviceValidationVersion() < VK_API_VERSION_1_1) {
-        GTEST_SKIP() << "At least Vulkan version 1.1 is required";
-    }
-
-    auto ray_query_features = LvlInitStruct<VkPhysicalDeviceRayQueryFeaturesKHR>();
-    auto ray_tracing_features = LvlInitStruct<VkPhysicalDeviceRayTracingPipelineFeaturesKHR>(&ray_query_features);
-    auto features2 = LvlInitStruct<VkPhysicalDeviceFeatures2>(&ray_tracing_features);
-    GetPhysicalDeviceFeatures2(features2);
-
-    if (ray_tracing_features.rayTracingPipeline == VK_FALSE) {
-        printf("%s rayTracingPipeline feature not supported, skipping tests\n", kSkipPrefix);
-        return;
-    }
-
-    ray_query_features.rayQuery = VK_FALSE;
-
-    ASSERT_NO_FATAL_FAILURE(InitState(nullptr, &features2));
-
-    auto vkGetAccelerationStructureBuildSizesKHR = reinterpret_cast<PFN_vkGetAccelerationStructureBuildSizesKHR>(
-        vk::GetInstanceProcAddr(instance(), "vkGetAccelerationStructureBuildSizesKHR"));
-    assert(vkGetAccelerationStructureBuildSizesKHR != nullptr);
-
-    auto build_info = LvlInitStruct<VkAccelerationStructureBuildGeometryInfoKHR>();
-    build_info.type = VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR;
-    uint32_t max_primitives_count;
-    auto build_sizes_info = LvlInitStruct<VkAccelerationStructureBuildSizesInfoKHR>();
-    vkGetAccelerationStructureBuildSizesKHR(device(), VK_ACCELERATION_STRUCTURE_BUILD_TYPE_HOST_OR_DEVICE_KHR, &build_info,
-                                            &max_primitives_count, &build_sizes_info);
-}
-
 TEST_F(VkPositiveLayerTest, TestSwapchainImageFenceWait) {
     TEST_DESCRIPTION("Test waiting on swapchain image with a fence.");
 
@@ -777,8 +676,7 @@ TEST_F(VkPositiveLayerTest, TestSwapchainImageFenceWait) {
 
     ASSERT_NO_FATAL_FAILURE(InitState(nullptr, nullptr, VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT));
     if (!InitSwapchain()) {
-        printf("%s Cannot create surface or swapchain, skipping test\n", kSkipPrefix);
-        return;
+        GTEST_SKIP() << "Cannot create surface or swapchain";
     }
 
     VkFenceObj fence;
@@ -792,9 +690,8 @@ TEST_F(VkPositiveLayerTest, TestSwapchainImageFenceWait) {
     vk::GetSwapchainImagesKHR(device(), m_swapchain, &swapchain_images_count, swapchain_images.data());
 
     uint32_t image_index = 0;
-    vk::AcquireNextImageKHR(device(), m_swapchain, std::numeric_limits<uint64_t>::max(), VK_NULL_HANDLE, fence_handle,
-                            &image_index);
-    vk::WaitForFences(device(), 1, &fence_handle, VK_TRUE, std::numeric_limits<uint64_t>::max());
+    vk::AcquireNextImageKHR(device(), m_swapchain, kWaitTimeout, VK_NULL_HANDLE, fence_handle, &image_index);
+    vk::WaitForFences(device(), 1, &fence_handle, VK_TRUE, kWaitTimeout);
 
     m_commandBuffer->begin();
 
@@ -838,6 +735,22 @@ TEST_F(VkPositiveLayerTest, TestSwapchainImageFenceWait) {
 TEST_F(VkPositiveLayerTest, EnumeratePhysicalDeviceGroups) {
     TEST_DESCRIPTION("Test using VkPhysicalDevice handles obtained with vkEnumeratePhysicalDeviceGroups");
 
+#ifdef __linux__
+    if (std::getenv("NODEVICE_SELECT") == nullptr)
+    {
+        // Currently due to a bug in MESA this test will fail.
+        // https://gitlab.freedesktop.org/mesa/mesa/-/commit/4588453815c58ec848b0ff6f18a08836e70f55df
+        //
+        // It's fixed as of v22.7.1:
+        // https://gitlab.freedesktop.org/mesa/mesa/-/tree/mesa-22.1.7/src/vulkan/device-select-layer
+        //
+        // To avoid impacting local users, skip this TEST unless NODEVICE_SELECT is specified.
+        // NODEVICE_SELECT enables/disables the implicit mesa layer which has illegal code:
+        // https://gitlab.freedesktop.org/mesa/mesa/-/blob/main/src/vulkan/device-select-layer/VkLayer_MESA_device_select.json
+        GTEST_SKIP();
+    }
+#endif
+
     SetTargetApiVersion(VK_API_VERSION_1_1);
 
     auto ici = GetInstanceCreateInfo();
@@ -850,7 +763,7 @@ TEST_F(VkPositiveLayerTest, EnumeratePhysicalDeviceGroups) {
     uint32_t physical_device_group_count = 0;
     vk::EnumeratePhysicalDeviceGroups(test_instance, &physical_device_group_count, nullptr);
     std::vector<VkPhysicalDeviceGroupProperties> device_groups(physical_device_group_count,
-                                                               {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GROUP_PROPERTIES});
+                                                               LvlInitStruct<VkPhysicalDeviceGroupProperties>());
     vk::EnumeratePhysicalDeviceGroups(test_instance, &physical_device_group_count, device_groups.data());
 
     if (physical_device_group_count > 0) {
@@ -863,3 +776,136 @@ TEST_F(VkPositiveLayerTest, EnumeratePhysicalDeviceGroups) {
     debug_reporter.Destroy(test_instance);
     vk::DestroyInstance(test_instance, nullptr);
 }
+
+#ifdef VK_USE_PLATFORM_METAL_EXT
+TEST_F(VkPositiveLayerTest, ExportMetalObjects) {
+    TEST_DESCRIPTION("Test vkExportMetalObjectsEXT");
+
+    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_METAL_OBJECTS_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME);
+    if (!InstanceExtensionSupported(VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME)) {
+        GTEST_SKIP() << VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME << " not supported";
+    }
+
+    // Initialize framework
+    {
+        auto queue_info = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>();
+        queue_info.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_COMMAND_QUEUE_BIT_EXT;
+
+        auto metal_info = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>();
+        metal_info.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_DEVICE_BIT_EXT;
+        metal_info.pNext = &queue_info;
+
+        ASSERT_NO_FATAL_FAILURE(InitFramework(nullptr, &metal_info));
+        if (!AreRequiredExtensionsEnabled()) {
+            GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+        }
+
+        auto portability_features = LvlInitStruct<VkPhysicalDevicePortabilitySubsetFeaturesKHR>();
+        auto features2 = GetPhysicalDeviceFeatures2(portability_features);
+
+        ASSERT_NO_FATAL_FAILURE(InitState(nullptr, &features2));
+    }
+
+    auto vkExportMetalObjectsEXT =
+        reinterpret_cast<PFN_vkExportMetalObjectsEXT>(vk::GetDeviceProcAddr(m_device->device(), "vkExportMetalObjectsEXT"));
+    ASSERT_TRUE(vkExportMetalObjectsEXT != nullptr);
+
+    const VkDevice device = m_device->device();
+
+    // Get Metal Device and Metal Command Queue in 1 call
+    {
+        const VkQueue queue = m_device->m_queue;
+
+        auto queueInfo = LvlInitStruct<VkExportMetalCommandQueueInfoEXT>();
+        queueInfo.queue = queue;
+        auto deviceInfo = LvlInitStruct<VkExportMetalDeviceInfoEXT>(&queueInfo);
+        auto objectsInfo = LvlInitStruct<VkExportMetalObjectsInfoEXT>(&deviceInfo);
+
+        // This tests both device, queue, and pNext chaining
+        vkExportMetalObjectsEXT(device, &objectsInfo);
+
+        ASSERT_TRUE(deviceInfo.mtlDevice != nullptr);
+        ASSERT_TRUE(queueInfo.mtlCommandQueue != nullptr);
+    }
+
+    // Get Metal Buffer
+    {
+        auto metalBufferCreateInfo = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>();
+        metalBufferCreateInfo.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT;
+
+        VkMemoryAllocateInfo mem_info = LvlInitStruct<VkMemoryAllocateInfo>();
+        mem_info.allocationSize = 1024;
+        mem_info.pNext = &metalBufferCreateInfo;
+
+        VkDeviceMemory memory;
+        const VkResult err = vk::AllocateMemory(device, &mem_info, NULL, &memory);
+        ASSERT_VK_SUCCESS(err);
+
+        auto bufferInfo = LvlInitStruct<VkExportMetalBufferInfoEXT>();
+        bufferInfo.memory = memory;
+        auto objectsInfo = LvlInitStruct<VkExportMetalObjectsInfoEXT>(&bufferInfo);
+
+        vkExportMetalObjectsEXT(device, &objectsInfo);
+
+        ASSERT_TRUE(bufferInfo.mtlBuffer != nullptr);
+
+        vk::FreeMemory(device, memory, nullptr);
+    }
+
+    // Get Metal Texture and Metal IOSurfaceRef
+    {
+        auto metalSurfaceInfo = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>();
+        metalSurfaceInfo.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_IOSURFACE_BIT_EXT;
+        auto metalTextureCreateInfo = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>(&metalSurfaceInfo);
+        metalTextureCreateInfo.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_TEXTURE_BIT_EXT;
+
+        // Image contents don't matter
+        VkImageCreateInfo ici = LvlInitStruct<VkImageCreateInfo>(&metalTextureCreateInfo);
+        ici.imageType = VK_IMAGE_TYPE_2D;
+        ici.format = VK_FORMAT_B8G8R8A8_UNORM;
+        ici.extent = {32, 32, 1};
+        ici.mipLevels = 1;
+        ici.arrayLayers = 1;
+        ici.samples = VK_SAMPLE_COUNT_1_BIT;
+        ici.tiling = VK_IMAGE_TILING_LINEAR;
+        ici.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
+        VkImageObj image(m_device);
+        image.Init(ici);
+        ASSERT_TRUE(image.initialized());
+
+        auto surfaceInfo = LvlInitStruct<VkExportMetalIOSurfaceInfoEXT>();
+        surfaceInfo.image = image.handle();
+        auto textureInfo = LvlInitStruct<VkExportMetalTextureInfoEXT>(&surfaceInfo);
+        textureInfo.image = image.handle();
+        textureInfo.plane = VK_IMAGE_ASPECT_PLANE_0_BIT;  // Image is not multi-planar
+        auto objectsInfo = LvlInitStruct<VkExportMetalObjectsInfoEXT>(&textureInfo);
+
+        // This tests both texture, surface, and pNext chaining
+        vkExportMetalObjectsEXT(device, &objectsInfo);
+
+        ASSERT_TRUE(textureInfo.mtlTexture != nullptr);
+        ASSERT_TRUE(surfaceInfo.ioSurface != nullptr);
+    }
+
+    // Get Metal Shared Event
+    {
+        auto metalEventCreateInfo = LvlInitStruct<VkExportMetalObjectCreateInfoEXT>();
+        metalEventCreateInfo.exportObjectType = VK_EXPORT_METAL_OBJECT_TYPE_METAL_SHARED_EVENT_BIT_EXT;
+
+        auto eventCreateInfo = LvlInitStruct<VkEventCreateInfo>(&metalEventCreateInfo);
+        VkEventObj event(*m_device, eventCreateInfo);
+        ASSERT_TRUE(event.initialized());
+
+        auto eventInfo = LvlInitStruct<VkExportMetalSharedEventInfoEXT>();
+        eventInfo.event = event.handle();
+        auto objectsInfo = LvlInitStruct<VkExportMetalObjectsInfoEXT>(&eventInfo);
+
+        vkExportMetalObjectsEXT(device, &objectsInfo);
+
+        ASSERT_TRUE(eventInfo.mtlSharedEvent != nullptr);
+    }
+}
+#endif  // VK_USE_PLATFORM_METAL_EXT
