@@ -425,6 +425,12 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
   public:
     explicit Buffer() : NonDispHandle() {}
     explicit Buffer(const Device &dev, const VkBufferCreateInfo &info) { init(dev, info); }
+    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props) {
+        init(dev, info, mem_props);
+    }
+    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props, void *alloc_info_pnext) {
+        init(dev, info, mem_props, alloc_info_pnext);
+    }
     explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, NoMemT) { init_no_mem(dev, info); }
     explicit Buffer(const Device &dev, VkDeviceSize size) { init(dev, size); }
 
@@ -676,8 +682,8 @@ class AccelerationStructure : public internal::NonDispHandle<VkAccelerationStruc
 
     const VkDevice &dev() const { return device(); }
 
-    void create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo = NULL,
-                               bool buffer_device_address = false);
+    [[nodiscard]] vk_testing::Buffer create_scratch_buffer(const Device &device, VkBufferCreateInfo *pCreateInfo = nullptr,
+                                                           bool buffer_device_address = false) const;
 
   private:
     VkAccelerationStructureInfoNV info_;
@@ -700,8 +706,8 @@ class AccelerationStructureKHR : public internal::NonDispHandle<VkAccelerationSt
 
     const VkDevice &dev() const { return device(); }
 
-    void create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo = NULL,
-                               bool buffer_device_address = false);
+    [[nodiscard]] vk_testing::Buffer create_scratch_buffer(const Device &device, const VkBufferCreateInfo *pCreateInfo = nullptr,
+                                                           bool buffer_device_address = false) const;
 
   private:
     VkAccelerationStructureCreateInfoKHR info_;
@@ -752,9 +758,12 @@ class PipelineLayout : public internal::NonDispHandle<VkPipelineLayout> {
   public:
     PipelineLayout() noexcept : NonDispHandle() {}
     PipelineLayout(const Device &dev, VkPipelineLayoutCreateInfo &info,
-                   const std::vector<const DescriptorSetLayout *> &layouts = {}) {
+                   const std::vector<const DescriptorSetLayout *> &layouts) {
         init(dev, info, layouts);
     }
+    PipelineLayout(const Device &dev, VkPipelineLayoutCreateInfo &info) {
+        init(dev, info);
+    }
     ~PipelineLayout() noexcept;
 
     // Move constructor for Visual Studio 2013
@@ -768,6 +777,7 @@ class PipelineLayout : public internal::NonDispHandle<VkPipelineLayout> {
 
     // vCreatePipelineLayout()
     void init(const Device &dev, VkPipelineLayoutCreateInfo &info, const std::vector<const DescriptorSetLayout *> &layouts);
+    void init(const Device &dev, VkPipelineLayoutCreateInfo &info);
 };
 
 class Sampler : public internal::NonDispHandle<VkSampler> {
@@ -1187,6 +1197,51 @@ inline VkCommandBufferAllocateInfo CommandBuffer::create_info(VkCommandPool cons
     return info;
 }
 
+struct GraphicsPipelineLibraryStage {
+    layer_data::span<const uint32_t> spv;
+    VkShaderModuleCreateInfo shader_ci;
+    VkPipelineShaderStageCreateInfo stage_ci;
+
+    GraphicsPipelineLibraryStage(layer_data::span<const uint32_t> spv, VkShaderStageFlagBits stage = VK_SHADER_STAGE_VERTEX_BIT,
+                                 const char *name = "main")
+        : spv(spv) {
+        shader_ci = LvlInitStruct<VkShaderModuleCreateInfo>();
+        shader_ci.codeSize = spv.size() * sizeof(uint32_t);
+        shader_ci.pCode = spv.data();
+
+        stage_ci = LvlInitStruct<VkPipelineShaderStageCreateInfo>(&shader_ci);
+        stage_ci.stage = stage;
+        stage_ci.module = VK_NULL_HANDLE;
+        stage_ci.pName = name;
+    }
+};
+
+struct GraphicsPipelineFromLibraries {
+    layer_data::span<VkPipeline> libs;
+    VkPipelineLibraryCreateInfoKHR link_info;
+    vk_testing::Pipeline pipe;
+
+    GraphicsPipelineFromLibraries(const Device &dev, layer_data::span<VkPipeline> libs, VkGraphicsPipelineCreateInfo *ci = nullptr)
+        : libs(libs) {
+        link_info = LvlInitStruct<VkPipelineLibraryCreateInfoKHR>();
+        link_info.libraryCount = libs.size();
+        link_info.pLibraries = libs.data();
+
+        if (ci) {
+            link_info.pNext = ci->pNext;
+            ci->pNext = &link_info;
+            pipe.init(dev, *ci);
+        } else {
+            auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
+            pipe.init(dev, exe_pipe_ci);
+        }
+        pipe.initialized();
+    }
+
+    operator VkPipeline() const { return pipe.handle(); }
+    operator bool() const { return pipe.initialized(); }
+};
+
 }  // namespace vk_testing
 
 #endif  // VKTESTBINDING_H
